function model = Learning_MLE_Basis_My( Seqs, model, alg )

Aest = model.A;        
muest = model.mu;

if alg.LowRank
    UL = zeros(size(Aest));
    ZL = Aest;
end

if alg.Sparse
    US = zeros(size(Aest));
    ZS = Aest;
end

if alg.GroupSparse
    UG = zeros(size(Aest));
    ZG = Aest;
end

D = size(Aest, 1);

if alg.storeLL
    model.LL = zeros(alg.outer,1);
end
if alg.storeErr
    model.err = zeros(alg.outer, 3);
end

tic;

NLLouter = zeros(D, 1);
NLLinner = zeros(D, 1);

for o = 1:alg.outer
    
    rho = alg.rho * (1.1^o);
    

    for n = 1:alg.inner
        
        NLL = zeros(D, 1);

        Amu = zeros(D, 1);
        Bmu = Amu;
        
        
        CmatA = zeros(size(Aest));
        AmatA = CmatA;
        BmatA = CmatA;
        if alg.LowRank
            BmatA = BmatA + rho*(UL-ZL);
            AmatA = AmatA + rho;
        end
        if alg.Sparse
            BmatA = BmatA + rho*(US-ZS);
            AmatA = AmatA + rho;
        end
        if alg.GroupSparse
            BmatA = BmatA + rho*(UG-ZG);
            AmatA = AmatA + rho;
        end
        
        for c = 1:length(Seqs)
            if ~isempty(Seqs(c).Time)
                Time = Seqs(c).Time;
                Event = Seqs(c).Mark;
                Tstart = Seqs(c).Start;

                if isempty(alg.Tmax)
                    Tstop = Seqs(c).Stop;
                else
                    Tstop = alg.Tmax;
                    indt = Time < alg.Tmax;
                    Time = Time(indt);
                    Event = Event(indt);
                end

                
                Amu = Amu + Tstop - Tstart;

                dT = Tstop - Time;
                GK = Kernel_Integration(dT, model);

                Nc = length(Time);

                
                i = 1;
                while i <= Nc
                    i2 = i + 1;
                    while i2 <= Nc && Time(i2) == Time(i)
                        i2 = i2 + 1;
                    end
                    i2 = i2 - 1;
                    for ui=Event(i:i2)                        
                        BmatA(ui,:,:) = BmatA(ui,:,:)+...
                            double(Aest(ui,:,:)>0).*repmat( GK(i,:), [1,1,D] );
                        
                        ti = Time(i);
                        
                        lambdai = muest(ui);
                        pii = muest(ui);
                        pij = [];
                        
                        
                        if i>1
                            
                            tj = Time(1:i-1);
                            uj = Event(1:i-1);
                            
                            
                            dt = ti - tj;
                            gij = Kernel(dt, model);
                            
                            auiuj = Aest(uj, :, ui);
                            pij = auiuj .* gij;
                            lambdai = lambdai + sum(pij(:));
                        end
                        
                        NLL(ui) = NLL(ui) - log(lambdai);
                        pii = pii./lambdai;
                        
                        if i>1
                            pij = pij./lambdai;
                            if ~isempty(pij) && sum(pij(:))>0
                                for j = 1:length(uj)
                                    uuj = uj(j);
                                    CmatA(uuj,:,ui) = CmatA(uuj,:,ui) - pij(j,:);
                                end
                            end
                        end
                        
                        Bmu(ui) = Bmu(ui) + pii;
                    end
                    i = i2 + 1;
                end

                NLL = NLL + (Tstop-Tstart)*muest;
                NLL = NLL + squeeze(sum(sum(repmat(GK, 1, 1, D).*Aest(Event,:,:), 1), 2));
            
            else
                warning('Sequence %d is empty!', c)
            end
        end
                
        mu = Bmu./Amu;        
        if alg.Sparse==0 && alg.GroupSparse==0 && alg.LowRank==0
            A = -CmatA./BmatA;
            A(isnan(A))=0;
            A(isinf(A))=0;
        else            
            A = ( -BmatA + sqrt(BmatA.^2 - 4*AmatA.*CmatA) )./(2*AmatA);
            A(isnan(A))=0;
            A(isinf(A))=0;
        end
        
        if isfield(alg, 'target')
            target = alg.target;
            Err = abs(NLL(target) - NLLinner(target));
            Scale =  abs(NLLinner(target));
            Obj = NLL(target);
        else
            Err = abs(sum(NLL) - sum(NLLinner));
            Scale =  abs(sum(NLLinner));
            Obj = sum(NLL);
        end

        Aest = A;
        muest = mu;
        model.A = Aest;
        model.mu = muest;
        fprintf('Outer=%d, Inner=%d, Obj=%f, RelErr=%f, Time=%0.2fsec\n',...
                o, n, Obj, Err / Scale, toc);
        if Err < alg.thres*Scale || (o==alg.outer && n==alg.inner)
            break;
        end
        NLLinner = NLL;
    end
    
    if isfield(alg, 'target')
        target = alg.target;
        Err = abs(NLL(target) - NLLouter(target));
        Scale =  abs(NLLouter(target));
    else
        Err = abs(sum(NLL) - sum(NLLouter));
        Scale =  abs(sum(NLLouter));
    end
    if Err < alg.thres*Scale
        break;
    else
        NLLouter = NLL;
    end
    
    if alg.LowRank
        threshold = alg.alphaLR/rho;
        ZL = SoftThreshold_LR( Aest+UL, threshold );
        UL = UL + (Aest-ZL);
    end
    
    if alg.Sparse
        threshold = alg.alphaS/rho;
        ZS = SoftThreshold_S( Aest+US, threshold );
        US = US + (Aest-ZS);
    end

    if alg.GroupSparse
        threshold = alg.alphaGS/rho;
        ZG = SoftThreshold_GS( Aest+UG, threshold );
        UG = UG + (Aest-ZG);
    end

end




